/**
 * Copyright (C) 2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <libxfce4util/libxfce4util.h>

#include "thunar-lgfind.h"

/* thunar module */
static GType type_list[1];

G_MODULE_EXPORT void
thunar_extension_initialize (ThunarxProviderPlugin *plugin)
{
    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");

    ltop_search_register_type (plugin);
    type_list[0] = LTOP_TYPE_SEARCH;
}

G_MODULE_EXPORT void
thunar_extension_shutdown (void)
{
}

G_MODULE_EXPORT void
thunar_extension_list_types (const GType **types, gint *n_types)
{
    *types = type_list;
    *n_types = G_N_ELEMENTS (type_list);
}



/* thunar provider */
static void   ltop_search_menu_provider_init (ThunarxMenuProviderIface *iface);
static GList *ltop_search_get_file_actions (ThunarxMenuProvider      *provider,
                                            GtkWidget                *window,
                                            GList                    *list);

struct _LtopSearchClass
{
    GObjectClass __parent__;
};

struct _LtopSearch
{
    GObject         __parent__;
};

THUNARX_DEFINE_TYPE_WITH_CODE (LtopSearch, ltop_search, G_TYPE_OBJECT,
                               THUNARX_IMPLEMENT_INTERFACE (THUNARX_TYPE_MENU_PROVIDER,
                                                            ltop_search_menu_provider_init));

static void
ltop_search_class_init (LtopSearchClass *klass)
{
}

static void
ltop_search_init (LtopSearch *ltop_search)
{
}

static void
ltop_search_menu_provider_init (ThunarxMenuProviderIface *iface)
{
    iface->get_file_actions = ltop_search_get_file_actions;
}

static void
ltop_search_activated (GtkAction *action, gpointer data)
{
    gchar *tmp = NULL, *dirname = NULL;

    tmp = thunarx_file_info_get_uri ((ThunarxFileInfo *) data);
    dirname = g_filename_from_uri (tmp, NULL, NULL);
    g_free (tmp);

    tmp = g_strdup_printf ("lgfind -d %s", dirname);
    g_spawn_command_line_async (tmp, NULL);

    g_free (tmp);
    g_free (dirname);
}


static GList *ltop_search_get_file_actions (ThunarxMenuProvider      *provider,
                                            GtkWidget                *window,
                                            GList                    *list)
{
    GtkAction *action;

    if (g_list_length (list) != 1)
        return NULL;

    if (! thunarx_file_info_is_directory (list->data))
        return NULL;

    action = g_object_new (GTK_TYPE_ACTION, "name", "LopSearch::find", "label", "Find here...", NULL);
    g_signal_connect (G_OBJECT (action), "activate",
                      G_CALLBACK (ltop_search_activated), list->data);
    return g_list_append (NULL, action);
}

