/**
 * Copyright (C) 2009 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <gtk/gtk.h>
#include <libxfcegui4/libxfcegui4.h>

#ifndef MAIN_H
#define MAIN_H

#define CONFFILE "/cdrom/update.cfg"

typedef struct status {
    gchar *error;
    int count;
    GHashTable *images;
} status;

typedef struct token {
    gchar *error;
    gchar *token;
} token;

GHashTable *cfg;

/* helpers */
gint str_cmp_func (GtkTreeModel *model,
                   GtkTreeIter *a,
                   GtkTreeIter *b,
                   gpointer data);

const gchar *get_os ();

void parse_status_page (gchar *buffer, status *s);
void parse_prepupdate_page (gchar *buffer, token *t);
gboolean parse_cfg_page (gchar *buffer);

gboolean config_is (const gchar *key, const gchar *value);

gboolean get_prep_is_done ();
gboolean set_prep (gboolean prep);

gboolean save_config (const gchar *firstkey, ...);

void warn_service_down ();
void warn_setup_error ();

const gchar *get_selected_image (GtkComboBox *combo);

/* prototypes */
void login (gboolean mandatory);
status *get_informations (const gchar *login,
                          const gchar *passwd,
                          const gchar *os);
token *get_token (const gchar *login,
                  const gchar *passwd,
                  const gchar *imageid);
gboolean get_cfg ();
void update_dialog (status *s,
                    const gchar *login,
                    const gchar *passwd);

#endif
